#-*- coding: utf-8 -*-

#  
#  Copyright 2014 Jérôme Colin, Jérôme Kerdreux, Philippe Tanguy, Telecom Bretagne.
#  
#  This file is part of xAAL.
#  
#  xAAL is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  		
#  xAAL is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#  				
#  You should have received a copy of the GNU Lesser General Public License
#  along with xAAL. If not, see <http://www.gnu.org/licenses/>.
#

import datetime

from collections import deque

from xAAL.Devices import XAALDevice
from xAAL.tools import runActions 
from xAAL.Core import XAALEngine
from xAAL.Schemas import SchemasFactory

class CacheDevices(object):

    def __init__(self):
        self.deviceList = {}
    
    def update(self,deviceAddr=None,msg=None):
        if deviceAddr in self.deviceList.keys():
            data = self.deviceList[deviceAddr]  
            if data:
                body = msg.getBody()
                if body:
                    for key in body:
                        data[key] = body[key]
                data['timestamp'] = str(datetime.datetime.utcnow()) # update timestamp
                self.setDevice(deviceAddr=deviceAddr,data=data)
        else:
            data={}
            header = msg.getHeader()
            data['devType'] = header['devtype'] 
            data['address'] = deviceAddr 
            body = msg.getBody()
            if body:
                for key in body:
                    data[key] = body[key]
            self.setDevice(deviceAddr=deviceAddr,data=data)

    def setDevice(self,deviceAddr=None,data=None):
        self.deviceList[deviceAddr] = data 

    def getDevice(self,deviceAddr=None):
        if deviceAddr in self.deviceList.keys():
            return self.deviceList[deviceAddr]
        else:
            return None
    
    def getAll(self):
        return self.deviceList
    
    def isNewDevice(self,deviceAddr=None):
        if deviceAddr in self.deviceList.keys():
            return True
        else:
            return False


class XAALBrowser(XAALDevice):
    
    def __init__(self,address,bus=None):
        XAALDevice.__init__(self,"ihm.experimental",address)
        self.setVendorID("IHSEV")
        self.setProductID("User interface")
        self.cacheDevices = CacheDevices()
        self.engTx = XAALEngine()
        self.xaalSchema = SchemasFactory()
        self.bus = bus

    def send(self,target=None,action=None,msgtype=None):
        #get engine
        # Use other engine: Todo without...???
        #self.engTx.send(target=target,action=action,msgtype=msgtype)
        self.engTx.send(dev=self,target=target,action=action,msgtype=msgtype)
    
    def getDeviceList(self):
        devices = self.cacheDevices.getAll()
        return devices
    
    def getDevice(self,deviceAddr=None):
        device = self.cacheDevices.getDevice(deviceAddr=deviceAddr)
        if device:
            return device
        else:
            return None
 
    def getSchema(self,name):
        """Temporary method to get schema because we did not yet implement a xaal schema device"""
        return self.xaalSchema.get(name=name)

    ##########################
    #
    # callbacks
    #
    #########################    
    def callBack(self,msg,dev):
        r = None
        msg.dump()
        if msg.getMsgtype()=='notify':
            if msg.getAction()=='alive' and not msg.getSource() == self.getAddress():
                if not self.cacheDevices.isNewDevice(msg.getSource()):
                    # Todo: avoid engTx
                    self.send(target=msg.getSource(),action='getDescription',msgtype='request')
            elif msg.getAction()=='attributesChange':
                self.bus.publish('attributesChange',msg)
                #self.cacheDevices.update(deviceAddr=msg.getSource(),msg=msg)
        elif msg.getMsgtype()=='reply':
            if msg.getAction()=='getDescription':
                self.cacheDevices.update(deviceAddr=msg.getSource(),msg=msg)
                self.bus.publish('alive',msg)
            elif msg.getAction()=='getDevicesDesc':
                #Todo: avoid lines below. Change xAAL gateway getDevicesDesc...
                body = msg.getBody()
                newBody={}
                newBody['devices'] = body
                msg.setBody(newBody)
                self.cacheDevices.update(deviceAddr=msg.getSource(),msg=msg)
            elif msg.getAction()=='isAlive':
                self.cacheDevices.update(deviceAddr=msg.getSource(),msg=msg)
            elif msg.getAction() == 'getAttributes':
                self.bus.publish('attributesChange',msg)
        else:
            r = runActions(msg,dev)
        
        if r:
            return r
