# -*- coding: utf-8 -*-

#  
#  Copyright 2014 Jérôme Colin, Jérôme Kerdreux, Philippe Tanguy, Telecom Bretagne.
#  
#  This file is part of xAAL.
#  
#  xAAL is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  		
#  xAAL is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#  				
#  You should have received a copy of the GNU Lesser General Public License
#  along with xAAL. If not, see <http://www.gnu.org/licenses/>.
#  


from SitePage import SitePage
import cherrypy
try: import ujson as json
except ImportError: import json

        
class DeviceInfo(SitePage):
    def __init__(self,deviceAddr=None,xaalBrowser=None):
        SitePage.__init__(self,'DeviceInfo',xaalBrowser=xaalBrowser)
        self.addr = deviceAddr
        self.device =  self.xaalBrowser.getDevice(deviceAddr=deviceAddr)

    def getData(self):
        return self.device

    def getAddr(self):
        return self.addr
    
    def getEventName(self):
        return "'event_%s'" % str(self.addr) #quote because JS don't like numeric value in id ???
   
    def getDevType(self):
        data = self.getData()
        if data:
            devType = data['devType']
            return devType
    
    def getDefaultAttributes(self):
        schema = self.xaalBrowser.getSchema(name='any.any')
        return schema['attributes'].keys()
    
    def getSpecificAttributes(self):
        specificAttributes=list()
        schema = self.getSchema()
        allAttributes = schema['attributes'].keys()
        defaultAttributes = self.getDefaultAttributes()
        for attribute in allAttributes:
            if attribute not in defaultAttributes:
                specificAttributes.append(attribute)
        return specificAttributes
    
    def getDefaultMethods(self):
        schema = self.xaalBrowser.getSchema(name='any.any')
        return schema['methods'].keys()
    
    def getSpecificMethods(self):
        specificMethods=list()
        schema = self.getSchema()
        allMethods = schema['methods'].keys()
        defaultMethods = self.getDefaultMethods()
        for method in allMethods:
            if method not in defaultMethods:
                specificMethods.append(method)
        return specificMethods
    
    def getSchema(self):
        schema =  self.xaalBrowser.getSchema(name=self.getDevType())
        return schema
    
    def getSchemaJson(self):
        schema =  self.xaalBrowser.getSchema(name=self.getDevType())
        schemaJson = json.dumps(schema)
        return schemaJson
    
    def sendAction(self,action=None):
        self.xaalBrowser.send(target=self.addr,action=action,msgtype='request')
    
