# -*- coding: utf-8 -*-

#
#  Copyright 2014 Jérôme Colin, Jérôme Kerdreux, Telecom Bretagne.
#  
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  		
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  				
#  You should have received a copy of the GNU General Public License
#  along with this program. If not, see <http://www.gnu.org/licenses/>.
#  


import sys,os
import time
import ConfigParser

#Wii balance board
import cwiid
#xAAL
from xAAL.Devices import XAALGateway 
from xAAL.Core import XAALEngine
from xAAL.VariablesChange import VariablesChange
from xAAL.tools import runActions
from xAAL.Devices import XAALDevice


class WiiBalance(XAALDevice):
    def __init__(self,xaalAddr=None,wiiAddr=None):
        XAALDevice.__init__(self,devtype='weightscale.experimental',addr=xaalAddr,parent=None)
        self.setVendorID("IHSEV")
        self.setProductID("Wii balance board")
        self.__weight = VariablesChange('weight',self,default="unknown")
        self._wiiAddr = wiiAddr

    def start(self):
        print "-----Start WiiBalance--------"
        try:
            self.balance = cwiid.Wiimote(self._wiiAddr)
        except:
            print "Problem with the wii balance connection"
            quit()

        self.balance.rpt_mode = cwiid.RPT_BALANCE | cwiid.RPT_BTN #enable balance and button
        balance_cal = self.balance.get_balance_cal()
        self.balance_caldict = {'right_top': balance_cal[0],
            'right_bottom': balance_cal[1],
            'left_top': balance_cal[2],
            'left_bottom': balance_cal[3],
        }

    def setLed(self,state=None):
        if state == 'on':
            self.balance.led=1
        elif state == 'off':
            self.balance.led=0
   
    def readButton(self):
        if self.balance.state.has_key('buttons'):
            button = self.balance.state['buttons']
            if button:
                return True 
            else:
                return False
        else:
            print "No key buttons!!!"
            quit()
    
    def readWeight(self):
        if self.balance.state.has_key('balance'):
            data = self.balance.state['balance']
            myWeight = weight(data,self.balance_caldict)/100
        else:
            print "No key balance!!!"
            quit()
        if myWeight < 0:
            return -1
        else:
            return myWeight

    def setWeight(self,value):
        self.__weight.setValue(value)


    def getBatteryLevel(self):
        if self.balance.state.has_key('battery'):
            body = {"battery":self.balance.state['battery']}
            return body
        else:
            return None

    getBatteryLevel.exposed = True

    def callBack(self,msg,dev=None):
        r = None
        r = runActions(msg,dev)
        if r:
            return r


def weight(data,sensors_cal):
    weight = 0
    sensorsName = ('right_top', 'right_bottom', 'left_top', 'left_bottom')
    for name in sensorsName:
        val = data[name]
        cal = sensors_cal[name]
        if val < cal[1]: #???
            weight += 1700 * (val - cal[0]) / (cal[1] - cal[0]) #1700???
        else:
            weight += 1700 * (val - cal[1]) / (cal[2] - cal[1]) + 1700
    return weight

def main():
    balance = WiiBalance(xaalAddr='0b93a000-7f8d-4d8f-bb25-60bfa43d540a',wiiAddr='CC:9E:00:B5:3C:4A')
    balance.start()
    eng=XAALEngine()
    eng.registerDevice(balance)
    eng.registerCallBackHandler(balance.callBack)
    eng.start()

    isActive = False
    timeout = -1 
    while True:
        eng.loop()
        startButton = balance.readButton()
        if startButton:
            print "Balance activated"
            timeout = 500
        
        if timeout>0:
            weight = balance.readWeight()
            if weight == 0:
                balance.setLed('on')
                if not isActive:
                    balance.setWeight(weight)
                    isActive = True
            elif weight > 0 and isActive:
                balance.setWeight(weight)
            else:
                balance.setLed('off')
                isActive = False
            timeout = timeout - 1
        elif timeout==0:
            print "Balance sleeping"
            balance.setLed('off')
            timeout = - 1

if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        print "The Wii Balance device is now closed"

