#!/usr/bin/python2
# -*- coding: utf-8 -*-

#
#  Copyright 2014 Jérôme Colin, Jérôme Kerdreux, Telecom Bretagne.
#  
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  		
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  				
#  You should have received a copy of the GNU General Public License
#  along with this program. If not, see <http://www.gnu.org/licenses/>.
#  


MIXER_PADDING = 10


MIXER_PADDING = 10
SPOTIFY_SEND_CMD="dbus-send --print-reply --dest=org.mpris.MediaPlayer2.spotify /org/mpris/MediaPlayer2 org.mpris.MediaPlayer2.Player.%s"
SPOTIFY_QUERY_CMD="qdbus org.mpris.MediaPlayer2.spotify /org/mpris/MediaPlayer2 org.freedesktop.DBus.Properties.Get org.mpris.MediaPlayer2.Player PlaybackStatus"


import alsaaudio
import commands
from xAAL.Devices import XAALComposite,XAALDevice,XAALAttribute
from xAAL.Core import XAALEngine
from xAAL import tools



def getHostname():
    import platform
    return platform.node()


class SpotifyPlayer(XAALDevice):
   
    def __init__(self,address):
        XAALDevice.__init__(self,"mediaplayer.spotify",address)
        self.setVendorID("IHSEV")
        self.setProductID("Player spotify on %s" % getHostname())
        self.__state = XAALAttribute("state",self)
        

    def send(self,cmd):
        temp = SPOTIFY_SEND_CMD % cmd
        out = commands.getoutput(temp)
        print out

    def getStatus(self):
        return commands.getoutput(SPOTIFY_QUERY_CMD)

    #####################################################
    # Actions
    #####################################################    
    def play(self):
        s = self.getStatus()
        if s=='Paused':
            self.send('PlayPause')
        self.send('Play')
        self.__state.setValue('play')
    play.exposed=True
    
    def playUrl(self,url):
        self.send('OpenUri string:%s' % url)
    playUrl.exposed=True 
        
    def stop(self):
        self.send('Stop')
        self.__state.setValue('stop')
    stop.exposed=True
    
    def next(self):
        self.send('Next')
    next.exposed=True
 
    def prev(self):
        self.send('Previous')
    prev.exposed=True
    
    def pause(self):
        self.send('Pause')
        self.__state.setValue('pause')
    pause.exposed=True    


class Mixer(XAALDevice):
    
    def __init__(self,address):
        XAALDevice.__init__(self,"spotify.mixer",address)
        self.__mixer = alsaaudio.Mixer()
        self.__volume = XAALAttribute("volume",self)
        self.setVendorID("IHSEV")
        self.setProductID("Mixer")
        self.__last = None

    #####################################################
    # Actions
    #####################################################        
    def getVolume(self):
        vol = int(self.__mixer.getvolume()[0])
        return vol       
    getVolume.exposed=True
    
    def up(self):
        vol = self.getVolume() + MIXER_PADDING
        if vol > 100: vol = 100
        self.__mixer.setvolume(vol)
        self.__volume.setValue(vol)
    up.exposed=True
    
    def down(self):
        vol = self.getVolume() - MIXER_PADDING
        if vol < 0: vol = 0
        self.__mixer.setvolume(vol)
        self.__volume.setValue(vol)
    down.exposed=True
    
    def mute(self):
        self.__last = self.getVolume()
        self.__mixer.setvolume(0)
        self.__volume.setValue(0)
    mute.exposed=True        
    
    def unmute(self):
        if self.__last: 
            self.__mixer.setvolume(self.__last)
            self.__volume.setValue(self.__last)
    unmute.exposed=True


class SpotifyAgent(XAALComposite):
    
    def __init__(self,addrs={}):
        XAALComposite.__init__(self,addrs['agent'],None)
        self.setType("spotify.agent")
        self.setVendorID("IHSEV")
        self.setProductID("Agent Spotify")
        mix=Mixer(address=addrs['mixer'])
        player=SpotifyPlayer(addrs['player'])
        self.setDevices([mix,player])
        

    def callBack(self,msg,dev):
        r=tools.runActions(msg,dev)
        if r:
            return r




def main():
    configFile = tools.getConfigFile(SpotifyAgent.__name__)
    cfg = tools.loadConfigFile(configFile)

    if cfg == None:
        cfg = tools.createConfigFile(configFile)
        cfg.add_section('agent')
        cfg.set('agent','agentaddr',tools.getRandomUUID())
        cfg.set('agent','mixeraddr',tools.getRandomUUID())
        cfg.set('agent','playeraddr',tools.getRandomUUID())
        f = open(configFile,'w')
        cfg.write(f)

    addrs = {}
    addrs['agent']   = cfg.get('agent','agentaddr')
    addrs['mixer']   = cfg.get('agent','mixeraddr')
    addrs['player']  = cfg.get('agent','playeraddr')


    spot=SpotifyAgent(addrs)
    eng=XAALEngine()
    eng.setDevices([spot,] + spot.getDevices())
    eng.registerCallBackHandler(spot.callBack)

    eng.run()


if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        print "Bye bye"
