# -*- coding: utf-8 -*-

#  
#  Copyright 2014, Jérôme Colin, Jérôme Kerdreux, Philippe Tanguy, Telecom Bretagne.
#  
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
		
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
				
#  You should have received a copy of the GNU General Public License
#  along with this program. If not, see <http://www.gnu.org/licenses/>.


from xAAL.Devices import XAALGateway
from xAAL.Core import XAALEngine
from xAAL.tools import runActions
from KNXDevices import KNXPowerRelay
import KNX

import ConfigParser
import uuid
import sys



class KNXGateway(XAALGateway):
    
    def __init__(self,address=None):
        XAALGateway.__init__(self,address=address)
        self.setType('gateway.basic')
        try:
            self.__conn=KNX.KNXConnector()
        except:
            KNX.displayEIBDError()
            quit(1)

        self.setVendorID("IHSEV")
        self.setProductID("Gateway KNX")
    
    def sendKNX(self,devAddr,action):
        self.__conn.sendData(devAddr=devAddr,action=action)
   
    def callBack(self,msg,dev):
        r=None
        r=runActions(msg,dev)
        if r:
            return r
            
    def loop(self):  
        data =self.__conn.getData()
        if data:
            target=None
            knxAddr = data[7] 
            knxCmd = data[9]
            for dev in self.getDevices():
                #if knxAddr in [dev.getSendAddr(),dev.getRecvAddr()]:
                if knxAddr == dev.getRecvAddr():
                    target=dev
                    print "KNX Notif from :" , knxAddr
            if target:
                if knxCmd==KNX.POWER_ON_STATE:
                    target.setState("on")
                elif knxCmd==KNX.POWER_OFF_STATE:
                    target.setState("off")

def run(gw,eng):
    eng.start()                    
    while 1:
        eng.loop()
        gw.loop()    

def main():
    cfg = ConfigParser.RawConfigParser()
    configFile = 'knxconfig.cfg'
    cfg.read(configFile)
    eng=XAALEngine()

    try:
        addr = cfg.get('gateway','xaaladdr')
    except:
        addr = str(uuid.uuid4())
        if cfg.has_section('gateway'):
            print "Add addr %s to gateway" % addr
            cfg.set('gateway','xaaladdr',addr)
        else:
            print "Create gateway section and add xaaladdr in config file."
            cfg.add_section('gateway')
            cfg.set('gateway','xaaladdr',addr)
        
        cfg.write(open(configFile,'w'))
        cfg.read(configFile) # reload config structure

    gw = KNXGateway(address=addr)

    for section in cfg.sections():
        if cfg.has_option(section,'device'):
            if cfg.get(section,'device') == 'KNXPowerRelay':
                try:
                    addr = cfg.get(section,'xaaladdr')
                    print type(addr)
                except:
                    print "Add addr to %s" % section
                    addr = str(uuid.uuid4())
                    print type(addr)
                    cfg.set(section,'xaaladdr',addr)
                    cfg.write(open(configFile,'w'))
                    cfg.read(configFile) # reload config structure
               
                if cfg.has_option(section,'recvAddr') and cfg.has_option(section,'sendAddr'):
                    gw.addDevice(KNXPowerRelay(address=addr,
                        recvAddr=cfg.getint(section,'recvAddr'),
                        sendAddr=cfg.getint(section,'sendAddr'),
                        ))
                else:
                    print "Please verify knx address in config file: %s" % configFile
                    sys.exit(0)

    eng.setDevices([gw,] + gw.getDevices())
    eng.registerCallBackHandler(gw.callBack)

    run(gw,eng)

if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        print "Bye bye"
