# -*- coding: utf-8 -*-

#  
#  Copyright 2014 Jérôme Kerdreux Telecom Bretagne.
#  
#  This file is part of xAAL.
#  
#  xAAL is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  		
#  xAAL is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  				
#  You should have received a copy of the GNU General Public License
#  along with xAAL. If not, see <http://www.gnu.org/licenses/>.
#  


from xAAL.Devices import XAALGateway,XAALDevice,XAALAttribute
from xAAL.Core import XAALEngine
from xAAL import tools

import X10


class X10PowerRelay(XAALDevice):

    def __init__(self,controller,house_code,device_code,addr):
        XAALDevice.__init__(self,devtype='powerrelay.basic',addr=addr)
        self.__power=XAALAttribute('power',self,default="unknow")
        self.setProductID('AM12F')
        self.setVendorID('Xdom')
        self.__x10 = X10.Switch(controller,house_code,device_code)


    def on(self):
        self.__power.setValue('on')
        self.__x10.on()
    on.exposed = True

    def off(self):
        self.__power.setValue('off')
        self.__x10.off()
    off.exposed = True


class X10Lamp(XAALDevice):

    def __init__(self,controller,house_code,device_code,addr):
        XAALDevice.__init__(self,devtype='lamp.dimmer',addr=addr)
        self.__light=XAALAttribute('light',self,default="unknow")
        self.__dimmer=XAALAttribute('dimmer',self,default="unknow")
        self.setProductID('LM12')
        self.setVendorID('Xdom')
        self.__x10 = X10.Lamp(controller,house_code,device_code)
    

    def on(self):
        self.__light.setValue('on')
        self.__x10.on()
    on.exposed=True

    def off(self):
        self.__light.setValue('off')
        self.__x10.off()
    off.exposed=True

    def dim(self,value=50):
        self.__light.setValue('on')
        self.__dimmer.setValue(value)
        self.__x10.dim(value)
    dim.exposed=True


class X10Gateway(XAALGateway):
    
    def __init__(self,address,tty):
        XAALGateway.__init__(self,address=address)
        self.setType('gateway.basic')
        self.setVendorID("IHSEV")
        self.setProductID("X10-CM11 Gateway")
        self.__cm11 = X10.CM11()
        self.__cm11.open(tty)

    def getX10Controller(self):
        return self.__cm11
        

    def callBack(self,msg,dev):
        r=None
        r=tools.runActions(msg,dev)
        if r:
            return r
       


def loadConfig():
    configFile = tools.getConfigFile(X10Gateway.__name__)
    cfg = tools.loadConfigFile(configFile)

    if cfg == None:
        cfg = tools.createConfigFile(configFile)
        cfg.add_section('gateway')
        cfg.set('gateway','xaaladdr',tools.getRandomUUID())
        cfg.set('gateway','serialport', "/dev/ttyUSB0")
        f = open(configFile,'w')
        cfg.write(f)


    addr = cfg.get('gateway','xaaladdr')
    port = cfg.get('gateway','serialport')

    gw=X10Gateway(addr,port)       

    for section in cfg.sections():
        dev = None
        if cfg.has_option(section,'type'):
            addr = cfg.get(section,'xaaladdr')
            house = cfg.get(section,'house')
            unit = int(cfg.get(section,'unit'))
            dev_type = cfg.get(section,'type')
            print "New X10%s (%s/%s)" % (dev_type,house,unit)

            if dev_type == 'PowerRelay':
                dev = X10PowerRelay(gw.getX10Controller(),house,unit,addr)
            if dev_type == 'Lamp':
                dev = X10Lamp(gw.getX10Controller(),house,unit,addr)
        if dev:
            gw.addDevice(dev)
    return gw



def main():
    
    gw = loadConfig()

    eng=XAALEngine()
    eng.setDevices([gw,] + gw.getDevices())
    eng.registerCallBackHandler(gw.callBack)
    eng.run()
    
     

if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        print "Bye bye"
