# -*- coding: utf-8 -*-

#  
#  Copyright 2014 Jérôme Kerdreux, Philippe Tanguy, Telecom Bretagne.
#  
#  This file is part of xAAL.
#  
#  xAAL is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  		
#  xAAL is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#  				
#  You should have received a copy of the GNU Lesser General Public License
#  along with xAAL. If not, see <http://www.gnu.org/licenses/>.
#  

from xAAL.Devices import XAALDevice

class ZwaveDevice(XAALDevice):
    def __init__(self,manager=None,homeId=None,nodeId=None,devtype=None,xaalAddr=None):
        XAALDevice.__init__(self,devtype=devtype,addr=xaalAddr,parent=None)
        self.__network = manager 
        self._homeId = homeId
        self._nodeId = nodeId
        self._name = None
        self._location = None
        self._productName = None
        self._productType = None
        self.valueId = dict() # each value is inside valueID identified by its 'id'

    def getHomeId(self):
        return self._homeId

    def getNodeId(self):
        return self._nodeId

    def getNodeName(self):
        return self.__network.getNodeName(self._homeId,self._nodeId)
    
    def setNodeName(self,value):
        self.__network.setNodeName(self._homeId,self._nodeIdi,value)
    
    def getNodeLocation(self):
        return self.__network.getNodeLocation(self._homeId,self._nodeId)
    
    def setNodeLocation(self,value):
        self.__network.setNodeLocation(self._homeId,self._nodeId,value)
    
    def getNodeStat(self):
        return self.__network.getNodeStatistics(self._homeId,self._nodeId)

    def getValueId(self,commandClass=None, instance=None):
        valueId = None
        for valId,val in self.valueId.items():
            if val['commandClass']==commandClass and val['instance']==instance:
                    valueId = valId
        
        if valueId:           
            return valueId
        else:
            return None
    
    def setValueId(self, valueId):
        id = valueId['id']
        self.valueId[id] = valueId
    
    def removeValueId(self,id=None):
        if self.valueId.has_key(id):
            del self.valueId[id]

    def getValue(self,id=None):
        if id in self.valueId:
            return self.valueId[id]['value']
        else:
            return None
    
    def setValue(self,id=None,value=None):
        self.__network.setValue(id=id,value=value)
    
    def parseMsg(self,msg=None):
        notif = msg['notificationType']
        print notif
        if notif == "ValueAdded":
            self.setValueId(msg['valueId'])
        
        if notif == "ValueRemoved":
            self.removeValueId(msg['valueId']['id'])


