# -*- coding: utf-8 -*-

#  
#  Copyright 2014 Jérôme Kerdreux, Philippe Tanguy, Telecom Bretagne.
#  
#  This file is part of xAAL.
#  
#  xAAL is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  		
#  xAAL is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#  				
#  You should have received a copy of the GNU Lesser General Public License
#  along with xAAL. If not, see <http://www.gnu.org/licenses/>.
#  


import datetime

from xAAL import tools
from xAAL.Devices import XAALDevice
from xAAL.Core import XAALEngine
from xAAL.tools import runActions 
from collections import defaultdict
from time import time

class Memory():

    def __init__(self,timeout=0):
        self._timeout = timeout #timeout in seconds
        self._cache = {}

    def setTimeout(self,timeout):
        self._timeout = timeout

    def getTimeout(self):
        return self._timeout

    def set(self,key,values):
        if self._cache.has_key(key):
            self._cache[key]['values'] = values
            self._cache[key]['timeout'] = time() + self._timeout
        else:
            self._cache[key] = {'values': values,
                                'timeout': time()+self._timeout}

    def get(self,key):
        if self._cache.has_key(key):
            return self._cache[key]['values']
        else:
            return None

    def getAll(self):
        return self._cache

    def update(self,key):
        if self._cache.has_key(key):
            #update timeout
            self._cache[key]['timeout'] += self._timeout 
            print "restart timer for device: %s" % key

    def removeExpired(self):
        expiredKeys = [key for key,data in self._cache.iteritems() if data['timeout']<time()]
        if expiredKeys:
            for key in expiredKeys:
                del self._cache[key]
    
    def remove(self,key):
        if self._cache.has_key(key):
            del self._cache[key]        

    def clear(self):
        self._cache.clear()


class CacheDevice(XAALDevice):
    
    def __init__(self,address):
        XAALDevice.__init__(self,"db.Cache",address,None)
        self._cache = Memory(timeout=60)

    def cacheDevice(self,msg):
        body = msg.getBody()
        body["timestamp"] = str(datetime.datetime.utcnow())
        self._cache.set(msg.getSource(),body)
    
    def updateTimeoutDevice(self,msg):
        self._cache.update(msg.getSource())

    def deleteTimeoutDevice(self):
        self._cache.removeExpired()
    
    ########################
    #
    # Actions
    #
    #######################
    
    def clearCache(self):
        print "CLEAR CACHE"
        self._cache.clear()
    
    clearCache.exposed = True
    
   # def getCache(self):
   #     if self._cache.getAll():
   #         print self._cache.getAll()
   # 
   # getCache.exposed = True

    def getCacheWithDevice(self,address):
        values = self._cache.get(address)
        print values
        if values:
            body = values
            return body
    
    getCacheWithDevice.exposed = True

    ########################
    #
    # callbacks
    #
    #######################
    
    def callBack(self,msg,dev=None):
        r = None
        if msg.getMsgtype() == "notify" and msg.getAction() == "attributesChange":
            self.cacheDevice(msg)
        elif msg.getMsgtype() == "notify" and msg.getAction() == "alive":
            self.updateTimeoutDevice(msg) 
        else:
            r = runActions(msg,dev)

        if r:
            return r


def run(eng):
    eng.start()
    timer = tools.Timer(30)
    timer.start()
    device = eng.getDevices()[0]
    while 1:
        eng.loop()
        if timer.reached():
            device.deleteTimeoutDevice()
            timer.start()


cache = CacheDevice(address = 'ff57b403-d0c6-490e-86cc-4c6664223aab')
eng = XAALEngine()
eng.registerDevice(cache)
eng.registerCallBackHandler(cache.callBack)
run(eng)

