# -*- coding: utf-8 -*-

#  
#  Copyright 2014 Jérôme Kerdreux, Philippe Tanguy, Telecom Bretagne.
#  
#  This file is part of xAAL.
#  
#  xAAL is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  		
#  xAAL is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#  				
#  You should have received a copy of the GNU Lesser General Public License
#  along with xAAL. If not, see <http://www.gnu.org/licenses/>.
#  


from sqlobject import *
from sqlobject.sqlbuilder import *

from settings import connection


# Device and func to manage
class Device(SQLObject):
    address = IntCol(unique=True)
    tags = RelatedJoin('Tag',
            intermediateTable='device_tag',
            createRelatedTable=False)

def getDevices():
    try:
        devices = list(Device.select())
        return devices
    except Exception, e:
        print "%s" % e

def getDevice(address):
    device = list(Device.selectBy(address = address))
    try:
        return device[0]
    except IndexError:
        return []

def addDevice(address):
    try:
        device = Device(address = address)
    except Exception, e:
        print "%s" % e

def removeDevice(dev):
    dev.delete(dev.id)

# Tag and func to manage
class Tag(SQLObject):
    name = StringCol(unique=True)
    devices = RelatedJoin('Device',
            intermediateTable='device_tag',
            createRelatedTable=False)

def getTags():
        tags = list(Tag.select())
        return tags

def getTag(tagname):
    tag = Tag.selectBy(name = tagname)
    try:
        return tag[0]
    except IndexError:
        return [] 

def addTag(name):
    try:
        tag = Tag(name = name)
    except Exception, e:
        print "%s" % e

def removeTag(tag):
    tag.delete(tag.id)


# Intermediate Class for many to many relationship
class DeviceTag(SQLObject):
    class sqlmeta:
        table="device_tag"
    device = ForeignKey('Device', notNull=True, cascade=True)
    tag = ForeignKey('Tag', notNull=True, cascade=True)
    unique = index.DatabaseIndex(device,tag,unique=True)
            

# Func to manage tables
def createTables():
    for table in (Device, Tag, DeviceTag):
        table.createTable(ifNotExists=True)

def dropTables():
    for table in (Device, Tag, DeviceTag):
        table.dropTable(ifExists=True)
