# -*- coding: utf-8 -*-

#  
#  Copyright 2014 Jérôme Kerdreux, Philippe Tanguy, Telecom Bretagne.
#  
#  This file is part of xAAL.
#  
#  xAAL is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  		
#  xAAL is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#  				
#  You should have received a copy of the GNU Lesser General Public License
#  along with xAAL. If not, see <http://www.gnu.org/licenses/>.
#  


import models
import inspect

from xAAL.Devices import XAALDevice
from xAAL.Core import XAALEngine
from xAAL.tools import runActions 

class Metadata(XAALDevice):

    def __init__(self,address):
        XAALDevice.__init__(self,"db.metadata",address,None)

    def getDevices(self):
        devices = models.getDevices()
        if devices:
            body= {"devices" : [dev.address for dev in devices] }
        else:
            body= {"devices" : [] }

        return body

    getDevices.exposed = True

    def getDevicesWithTag(self, name):
        tag = models.getTag(name)
        if tag:
            body= {"devicesWithTag" : [t.address for t in tag.devices] }
        else:
            body= {"devicesWithTag" : [] }

        return body

    getDevicesWithTag.exposed = True

    def getTags(self):
        tags = models.getTags()
        if tags:    
            body= {"tags" : [tag.name for tag in tags] }
        else:
            body= {"tags" : [] }

        return body

    getTags.exposed = True

    def getTagsWithDevice(self,address):
        device = models.getDevice(address)
        if device:
            body= {"tagsWithDevice" : [tag.name for tag in device.tags] }
        else:
            body= {"tagsWithDevice" : [] }

        return body

    getTagsWithDevice.exposed = True

    def addDevice(self, address):
        models.addDevice(address)

    addDevice.exposed = True

    def removeDevice(self, address):
        dev = models.getDevice(address)
        if dev:
            models.removeDevice(dev)
        else:
            print "No device to delete"

    removeDevice.exposed = True

    def addTag(self, name):
        models.addTag(name)

    addTag.exposed = True

    def removeTag(self, name):
        tag = models.getTag(name)
        if tag:
            models.removeTag(tag)
        else:
            print "No tag to delete"

    removeTag.exposed = True

    def addTagWithDevice(self, name, address):
        device = models.getDevice(address)
        if device:
            tag = models.getTag(name)
            if [tagtest for tagtest in device.tags if tagtest.name == name]:
                print "The Tag is already related to device"
            elif tag:
                device.addTag(tag.id)
                print "The tag has been added to device"
            else:
                models.addTag(name)
                tag = models.getTag(name)
                device.addTag(tag.id)
                print "The tag has been created and added to device"
        else:
            print "The device does not exist"

    addTagWithDevice.exposed = True

    def removeTagWithDevice(self, name, address):
        device = models.getDevice(address)
        tag = models.getTag(name)
        if device:
            if [tagtest for tagtest in device.tags if tagtest.name == name]:
                device.removeTag(tag)
                print "The tag has been removed"
            else:
                print "No need to remove this tag because it does not exist"
        else:
            print "The device does not exist"

    removeTagWithDevice.exposed = True

    def callBack(self,msg,dev=None):
        r = None
        r = runActions(msg,dev)
        if r:
            return r

metadata = Metadata(address='a828f690-2b91-48d6-b50a-74c56b678b3b')
eng = XAALEngine()
eng.registerDevice(metadata)
eng.registerCallBackHandler(metadata.callBack)
eng.run()
