# -*- coding: utf-8 -*-

#  
#  Copyright 2014, Jérôme Colin, Jérôme Kerdreux, Philippe Tanguy, Telecom Bretagne.
#  
#  This file is part of xAAL.
# 
#  xAAL is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#		
#  xAAL is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#				
#  You should have received a copy of the GNU Lesser General Public License
#  along with xAAL. If not, see <http://www.gnu.org/licenses/>.
#


import cjson
import config

class MessageFactoryParserError(Exception):pass
class MessageFactory():
           
    def encodeMSG(self,msg):
        if msg.getBody():
            txt={"msg":[msg.getHeader(),msg.getBody()]}
        else:
            txt={"msg":[msg.getHeader(),]}
        data=cjson.encode(txt)
        return data
        
    def decodeMSG(self,msg):
        data=cjson.decode(msg)
        return data
          
    def buildMSG(self,dev=None,target=config.XAAL_BCAST_ADDR,msgtype=None,action=None,body=None):
        """ the build method takes in parameters :\n
             -A device\n
             -the target of the message\n
             -The type of the message\n
             -The action of the message\n
             -A body if it's necessary (None if not)
             it will return a message encoded in Json.
        """
        message=XAALMessage()
        if dev:
            message.setSource(dev.getAddress())
            message.setDevtype(dev.getType())

        if target:
            message.setTarget(target)
        if msgtype:
            message.setMsgtype(msgtype)
        if action:
            message.setAction(action)
        message.setCipher('')
        message.setSignature('')
        
        if body!=None and body!={}:
            message.setBody(body)
        
        message.dump()
        data =self.encodeMSG(message)
        return data
         
    def buildAlivefor(self,dev):
        message=self.buildMSG(dev=dev,target=config.XAAL_BCAST_ADDR,msgtype="notify",action="alive")
        return message
        
    def buildErrorMSG(self,dev,errcode,description=None):
        message=XAALMessage()
        body={}
        body['errorCode']=errcode        
        if description: body['description']=description
        message=self.buildMSG(dev,config.XAAL_BCAST_ADDR,"notify","error",body)
        return message
        
    def parseMSG(self,buff):
        try:
           data =self.decodeMSG(buff)
        except cjson.DecodeError:
            raise MessageFactoryParserError,"Unable to parse JSON data"
         
        message=XAALMessage()
        head={}
        for (name,value) in data["msg"][0].items():
             head[name]=value
        
        message.setHeader(head)
        body={}
        if len(data["msg"])>1:
            for (name,value) in data["msg"][1].items():
                body[name]=value
                message.setBody(body)
        return message
         
  
class XAALMessage():
    
    def __init__(self):
        self.__header={}
        self.__body={}
        self.__header['version'] = config.STACK_VERSION


    def __getHeaderValue(self,key):
        if self.__header.has_key(key):
            return self.__header[key]
        return None

    #####################################################
    # Header attributes 
    #####################################################
    # devtype
    def setDevtype(self,value):
        self.__header['devtype']=value
        
    def getDevtype(self):
        return self.__getHeaderValue('devtype')

    # Source
    def setSource(self,value):
        self.__header['source']=value
    
    def getSource(self):
        return self.__getHeaderValue('source')
    
    # target
    def setTarget(self,value):
        self.__header['target']=value
    
    def getTarget(self):
        return self.__getHeaderValue('target')

    # msgtype
    def setMsgtype(self,value):
        self.__header['msgtype']=value
    
    def getMsgtype(self):
        return self.__getHeaderValue('msgtype')

    # Action
    def setAction(self,value):
        self.__header['action']=value
    
    def getAction(self):
        return self.__getHeaderValue('action')

    # version        
    def getVersion(self):
        return self.__getHeaderValue('version')
    
    # Cipher
    def setCipher(self,value=None):
        self.__header['cipher']=value
        
    def getCipher(self):
        return self.__getHeaderValue('cipher')
        
    # signature
    def setSignature(self,value=""):
        self.__header['signature']=value
    
    def getSignature(self):
        return self.__getHeaderValue('signature')

    #Parent
    def setParent(self,value):
        self.__header['parent']=value
        
    def getParent(self):
        return self.__getHeaderValue('parent')

    #####################################################
    # Header
    #####################################################
    def setHeader(self,head):
        self.__header=head
    
    def getHeader(self):
        return self.__header

    #####################################################
    # Body 
    #####################################################
    def setBody(self,body):
        self.__body=body
    
    def getBody(self):
        return self.__body

    def getParameters(self,args):
        body = self.getBody()
        parameters={}
        for p in range(0,len(args)):
            if isinstance(body[args[p]], str):
                param = "'%s'" % body[args[p]]
            else:
                param = body[args[p]]

            parameters[args[p]]= param

        return parameters

        
    #####################################################
    # Usefull methods 
    #####################################################
    def dump(self):
        print "========= %s =========" % self
        header = self.getHeader()
        print "*****Header*****\t"        
        if 'devtype' in header: print "Devtype\t%s"  % header['devtype']
        if 'action' in header: print "Action: \t%s"   % header['action']
        if 'msgtype' in header: print "Msgtype: \t%s"  % header['msgtype']
        if 'version' in header: print "Version: \t%s"  % header['version']
        if 'cipher' in header: print "Cipher: \t%s"   % header['cipher']
        if 'signature' in header: print "Signature: \t%s"% header['signature']
        if 'target' in header: print "Target: \t%s"   % header['target']
        if 'source' in header: print "Source: \t%s"   % header['source']
        if 'parent' in header: print "Parent: \t%s" % header['parent']
        
        if self.getBody():
            print "*****Body*****\t"
            print "%s \t" %self.getBody()
    
    
    #####################################################
    # Properties 
    #####################################################
    header=property(getHeader,setHeader)
    body=property(getBody,setBody)
    

