# -*- coding: utf-8 -*-

#  
#  Copyright 2014, Jérôme Colin, Jérôme Kerdreux, Philippe Tanguy, Telecom Bretagne.
#  
#  This file is part of xAAL.
# 
#  xAAL is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#		
#  xAAL is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#				
#  You should have received a copy of the GNU Lesser General Public License
#  along with xAAL. If not, see <http://www.gnu.org/licenses/>.
#

import config

import socket
import struct
import select


class NetworkConnector():

    def __init__(self,address=config.ADDR,port=config.PORT,ttl=config.TTL):
        self.connect(address,port,ttl)

    def connect(self,address=config.ADDR,port=config.PORT,ttl=config.TTL,bind_addr = ''):
        self.__address = address
        self.__port = port
        self.__sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
        #self.__sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
        self.__sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.__sock.bind((bind_addr, self.__port))
        mreq = struct.pack('4sl', socket.inet_aton(self.__address), socket.INADDR_ANY)
        self.__sock.setsockopt(socket.IPPROTO_IP, socket.IP_ADD_MEMBERSHIP, mreq)
        self.setTTL(ttl)


    def disconnect(self):
        self.__sock.close()

    def setTTL(self,value):
        self.__ttl = value
        self.__sock.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_TTL, value)
    
    def getTTL(self):
        return self.__ttl

    def receive(self):
        packt=self.__sock.recv(10240)
        return packt

    def getData(self):
        r = select.select([self.__sock,],[],[],0.1)
        if r[0]:
            return self.receive()
        return None

    def send(self,message):
        self.__sock.sendto(message, (self.__address, self.__port))            

    def close(self):
        self.__sock.close()
        
    def getAddr(self):
        return self.__address
        
    def getPort(self):
        return self.__port
    
    def setStackVersion(self,stackVersion):
        config.STACK_VERSION=stackVersion

    def getStackVersion(self):
        return config.STACK_VERSION
