# -*- coding: utf-8 -*-

#  
#  Copyright 2014, Jérôme Colin, Jérôme Kerdreux, Philippe Tanguy, Telecom Bretagne.
#  
#  This file is part of xAAL.
# 
#  xAAL is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#		
#  xAAL is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#				
#  You should have received a copy of the GNU Lesser General Public License
#  along with xAAL. If not, see <http://www.gnu.org/licenses/>.
#


import os
try: import ujson as json 
except ImportError: import json
from pprint import pprint
import config

def dump(jsonDict):
    print "=" * 80
    keys = jsonDict.keys()
    keys.sort()
    for k in keys:
        if k in ["notifications","attributes","methods"]:
            print "====== %s ======= " % k
        else:
            print "== %s => " % k,
        pprint(jsonDict[k])


class SchemasFactory:
    
    def __init__(self):
        self.__cache = {} 

    def load(self,filename):
        """ load schema from disk, and put it in cache
            return the file as dict"""
        if self.__cache.has_key(filename):
            print "Serving cache for %s" % filename
            return self.__cache[filename]

        print "Loading %s" % filename
        path = os.path.join(config.SCHEMA_DIR,filename)
        data = open(path,'r').read()
        jsonDict = json.loads(data)
        self.__cache.update({filename:jsonDict})
        return jsonDict

        
    def getExtends(self,name):
        """return the chain list off extends in reverse order, any.any is the first item"""
        current = name 

        extends = [name,]
        while 1:
            tmp = self.load(current)
            if tmp.has_key("extends"):
                current = tmp["extends"]
                extends.append(current)
            else:
                break
        extends.reverse()
        return extends


    def get(self,name):
        """return an complete schema w/ all extends included"""
        ext = self.getExtends(name)
        res = self.load(name)

        tmpMethods = {} 
        tmpAttr = {} 
        tmpNotifs = {}
        for e in ext:
            dict = self.load(e)

            if dict.has_key("methods"):
                tmp = dict["methods"]
                tmpMethods.update(tmp)
            
            if dict.has_key("attributes"):
                tmp = dict["attributes"]
                tmpAttr.update(tmp)

            if dict.has_key("notifications"):
                tmp = dict["notifications"]
                tmpNotifs.update(tmp)

        res["methods"] = tmpMethods
        res["attributes"] = tmpAttr
        res["notifications"] = tmpNotifs
        return res
        
