# -*- coding: utf-8 -*-

#  
#  Copyright 2014, Jérôme Colin, Jérôme Kerdreux, Philippe Tanguy, Telecom Bretagne.
#  
#  This file is part of xAAL.
# 
#  xAAL is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#		
#  xAAL is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#				
#  You should have received a copy of the GNU Lesser General Public License
#  along with xAAL. If not, see <http://www.gnu.org/licenses/>.
#


import time
import inspect
import config
import os
import ConfigParser
import uuid

class Timer:
    """ Simple Timer class"""

    def __init__(self,value=None):
        if value:
            self.set(value)

    def set(self,value):
        self.__timer = value

    def start(self):
        self.__last = time.time()

    def reached(self):
        now = time.time()
        if (now > (self.__last + self.__timer)):
            return True
        return False


#####################################################
#####################################################
def findActions(obj):
    """ loop throught the object to find and .exposed 
    functions attributes"""
    methodList = [e for e in dir(obj) if callable(getattr(obj, e))]
    result = []
    klass = obj.__class__ 
    for m in methodList:
        func = getattr(klass,m)
        if hasattr(func,'exposed'):
            if getattr(func,'exposed'):
                result.append(getattr(obj,m))
    return result


def filterMsgForDevices(msg,devices):
    """ loop throught the devices, to find which are 
    expected w/ the msg
    """
    results=[]   
    target = msg.getTarget()
    for dev in devices:
        if dev.getAddress() == target or target=='00000000-0000-0000-0000-000000000000':
            results.append(dev)
    return results


def runActions(msg,device):
    """ extract & run an action from an msg on the 
    selected device """
    if msg.getMsgtype()=="request":
        for action in device.getActionsMethod():
            if msg.getAction()== action.__name__:
                if msg.getBody():
                    args = getArgumentsForAction(action)
                    bodyParams = msg.getParameters(args)
                    params={}
                    for p in range(0,len(args)):
                        params[args[p]] = bodyParams[args[p]]
                        r = action(**params)
                else:
                    r= action()
                return r


def getArgumentsForAction(action):
    args = inspect.getargspec(action)
    try:
        args.args.remove('self')
    except:
        pass
    return args.args


def getConfigFile(name):
    filename = '%s.conf' % name
    return os.path.join(config.CONF_DIR,filename)


def loadConfigFile(filename):
    if os.path.exists(filename):
        cfg = ConfigParser.RawConfigParser()
        print "Loading config : %s" % filename
        cfg.read(filename)
        return cfg
    return None


def createConfigFile(filename):
    print "Empty config file %s" % filename 
    cfg = ConfigParser.RawConfigParser()
    return cfg


def getRandomUUID():
    return uuid.uuid1().__str__()
